# Copyright (C) 2010 Paraguay Educa, Martin Abente
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  US.

import gtk
import gobject
import logging
from gettext import gettext as _

from jarabe.model.session import get_session_manager

from sugar.graphics import style
from sugar.graphics.alert import Alert
from sugar.graphics.icon import Icon

from jarabe.controlpanel.sectionview import SectionView

from model import JournalManagementModel

class JournalManagement(SectionView):
    def __init__(self, model, alerts):
        SectionView.__init__(self)

        self._model = JournalManagementModel()
        self._model.connect('journal-management-running', self._set_update_log)
        self._model.connect('journal-management-started', self._set_status_started)
        self._model.connect('journal-management-finished', self._set_status_finished)
        self._model.connect('journal-management-failed', self._set_status_failed)

        self.restart_alerts = alerts
        self._showing_alert = False
        self._process_name = None
        self._needs_restart = True

        self.set_border_width(style.DEFAULT_SPACING)
        self.set_spacing(style.DEFAULT_SPACING)
        self._group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)

        self._vbox = gtk.VBox()

        self._setup_options()
        self._setup_status_info()
        self._setup_more_info()

        self._vbox.show()
        self.pack_start(self._vbox, expand=True)

    def _setup_options(self):
        self._setup_separator(_('Options'))

        self._setup_option(_('Backup'), \
            _('Save your Journal to the School Server'), \
                self._pre_start_backup_cb)

        self._setup_option(_('Restore'), \
            _('Restore your Journal to the last backup'), \
                self._pre_start_restore_cb)

    def _setup_option(self, button_text, description_text, button_callback):
        option_hbox = gtk.HBox()

        button = gtk.Button(label=button_text)
        button.connect('pressed', button_callback)
        button.show()
        option_hbox.pack_start(button, expand=True, fill=True)

        label = gtk.Label(description_text)
        label.show()
        option_hbox.pack_start(label, expand=True, fill=True)

        option_hbox.show()
        self._vbox.pack_start(option_hbox, expand=False, padding=5)

    def _setup_expander(self):
        expander = gtk.Expander(_('Show'))
        expander.connect("notify::expanded", self._log_menu_control)
        expander.show()

        self._vbox.pack_start(expander, expand=False)

    def _setup_more_info(self):
        self._setup_separator(_('Log'))
        self._setup_expander()
        self._setup_log()

    def _setup_status_info(self):
        self._setup_separator(_('Status'))

        self._status_label = gtk.Label(_('No process is running'))
        self._status_label.show()

        self._vbox.pack_start(self._status_label, expand=False, fill=False)

    def _setup_log(self):
        self._log_vbox = gtk.VBox()

        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self._log_vbox.pack_start(scrolled_window, expand=True, padding= 10)

        self._log_view = gtk.TextView()
        self._log_view.set_editable(False)
        self._log_buffer = self._log_view.get_buffer()
        scrolled_window.add(self._log_view)

        self._vbox.pack_start(self._log_vbox, expand=True)

    def _setup_separator(self, label_text):
        separator_vbox = gtk.VBox()

        separator = gtk.HSeparator()
        separator.show()
        separator_vbox.pack_start(separator, expand=False)

        label = gtk.Label(label_text)
        label.set_alignment(0, 0)
        label.show()
        separator_vbox.pack_start(label, expand=False)

        separator_vbox.show()
        self._vbox.pack_start(separator_vbox, expand=False, padding=10)

    def _log_menu_control(self, expander, data= None):
        if expander.get_expanded():
            self._log_vbox.show_all()
        else:
            self._log_vbox.hide()

    def _pre_start_restore_cb(self, button, data= None):
        self._show_alert(_('Restore Warning!'), \
            _('You are about to delete all your journal, are you sure?'), \
                self._response_restore_cb)

    def _response_restore_cb(self, alert, response_id):
        if response_id == gtk.RESPONSE_APPLY:
            self._process_name = _('Restore')
            self._needs_restart = True
            self._model.do_restore()

        self._destroy_alert(alert)

    def _pre_start_backup_cb(self, button, data= None):
        self._show_alert(_('Backup Warning!'), \
            _('This process could take several minutes, are you sure?'), \
                self._response_backup_cb)

    def _response_backup_cb(self, alert, response_id):
        if response_id == gtk.RESPONSE_APPLY:
            self._process_name = _('Backup')
            self._needs_restart = False
            self._model.do_backup()

        self._destroy_alert(alert)

    def _set_update_log(self, model, data):
        self._log_buffer.insert(self._log_buffer.get_end_iter(), data)
        self._log_view.scroll_to_iter(self._log_buffer.get_end_iter(), 0)

    def _set_status_started(self, model, data=None):
        self._status_label.set_text(self._process_name + ' ' +\
            _('is running, please wait...'))

        self._hack_hide_toolbar_buttons()

    def _set_status_finished(self, model, data=None):
        self._status_label.set_text(self._process_name + ' ' + \
            _('has finished, please check the logs.'))

        if self._needs_restart == True:
            self._show_alert(self._process_name + ' ' + \
                _('Warning!'), \
                    _('The process has finished, Sugar will restart now.'), \
                        self._post_process_restart_cb, \
                            True)
        else:
            self._hack_show_toolbar_buttons()

    def _post_process_restart_cb(self, alert, responde_id):
        session_manager = get_session_manager()
        session_manager.logout()

    def _set_status_failed(self, model, data=None):
        self._status_label.set_text(self._process_name + ' ' +\
            _('has failed.'))

        self._hack_show_toolbar_buttons()

    def _show_alert(self, title, msg, response_listener_cb, only_yes= False):
        if self._showing_alert == False and \
            self._model._running == False:
                alert = Alert()
                alert.props.title = title
                alert.props.msg = msg

                if only_yes == False:
                    icon = Icon(icon_name='dialog-cancel')
                    alert.add_button(gtk.RESPONSE_CANCEL, _('No'), icon)
                    icon.show()

                icon = Icon(icon_name='dialog-ok')
                alert.add_button(gtk.RESPONSE_APPLY, _('Yes'), icon)
                icon.show()

                alert.connect('response', response_listener_cb)
                alert.show()

                self._vbox.pack_start(alert, False)
                self._vbox.reorder_child(alert, 0)
                self._showing_alert = True

    def _destroy_alert(self, alert):
        self._vbox.remove(alert)
        self._showing_alert = False

    def _hack_show_toolbar_buttons(self):
        panel_control = self.get_toplevel()
        panel_control._section_toolbar.cancel_button.show()
        panel_control._section_toolbar.accept_button.show()

    def _hack_hide_toolbar_buttons(self):
        panel_control = self.get_toplevel()
        panel_control._section_toolbar.cancel_button.hide()
        panel_control._section_toolbar.accept_button.hide()

